package dnet

import (
	"context"
	"crypto/ecdsa"
	"log/slog"
	"net"
)

// Listener ...
type Listener struct {
	logger      *slog.Logger
	tcpListener *net.TCPListener
	identity    *ecdsa.PrivateKey
}

// Accept implements net.Listener.
func (l *Listener) Accept() (net.Conn, error) {
	ctx := context.Background()

	conn, err := l.tcpListener.AcceptTCP()
	if err != nil {
		return nil, err
	}

	clientIdentity, sessionKey, err := AcceptHandshake(ctx, conn, l.identity)
	if err != nil {
		return nil, err
	}

	return NewConn(conn, sessionKey, &l.identity.PublicKey, clientIdentity), nil
}

// Close implements net.Listener.
func (l *Listener) Close() error {
	return l.tcpListener.Close()
}

// Addr implements net.Listener.
func (l *Listener) Addr() net.Addr {
	laddr := l.tcpListener.Addr().(*net.TCPAddr)

	return NewAddr(laddr.IP, laddr.Port, &l.identity.PublicKey)
}
